// Example Java Code for the lecture on Graph Theory in DM534 by Daniel Merkle Nov. 8, 2018.
// For simplicity, no Exception Handling is implemented.

public class AdjacencyMatMult {

    // Assume M and N are both square (size x size) matrices 
    public static int[][] multiplySquared(int[][] M, int[][] N) {

        int size = M.length;

	// In Java, all values are initialized to 0
	// for good measure, the initialization is kept here
        int[][] C = new int[size][size];
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
		C[i][j] = 0;
            }
        }

        for (int i = 0; i < size; i++) { 
            for (int j = 0; j < size; j++) { 
                for (int k = 0; k < size; k++) { 
                    C[i][j] += M[i][k] * N[k][j];
                }
            }
        }

        return C;
    }

    public static int[][] addSquared(int[][] M, int[][] N) {
	
        int size = M.length;
	
        int[][] C = new int[size][size];
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
		C[i][j] = 0;
            }
        }

        for (int i = 0; i < size; i++) { 
            for (int j = 0; j < size; j++) { 
		C[i][j] += M[i][j] * N[i][j];
	    }
	}

	return C;
    }


    public static void printMatrix(int[][] M) {
	for (int i = 0; i < M.length; i++) {
	    for (int j = 0; j < M[0].length; j++)
                System.out.print(String.format("%3d ",M[i][j]));
            System.out.println();
        }
	System.out.println();
    }

    public static void main(String[] args) {
	int[][] A = {{0,1,0,0,1,0},
		     {1,0,1,0,1,0},
		     {0,1,0,0,0,0},
		     {0,0,0,0,1,1},
		     {1,1,0,1,0,0},
		     {0,0,0,1,0,0}};

	int size = A.length;
	
	int[][] R = A.clone();

	System.out.println("Initial Matrix A :");
	printMatrix(A);

	for (int i = 2; i < 5; i++) {
	    System.out.println(i+"-th power of A :");
	    R = multiplySquared(A, R);
	    printMatrix(R);
	}
    }
}
